# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import, unicode_literals

MSG_HOST = 'ホスト名: %s'
MSG_USER = '実行ユーザ: %s'
MSG_ITEM = '  [%d] | %s'
MSG_SUB_MENU = '%s へ'
MSG_QUIT = '終了'
MSG_RETURN = '%s に戻る'
MSG_INPUT_NUM = '番号を入力してください (%d-%d): '
MSG_INPUT_ANY = '何かキーを押すとメニューに戻ります...'
MSG_CONFIRM_TITLE = '実行確認'
MSG_CONFIRM = '  %s を行います。'
MSG_CONFIRM_QUESTION = 'よろしいですか? (y/n) [n]: '
MSG_RUN_TITLE = '実行: %s'
