from __future__ import division, print_function, absolute_import, unicode_literals

MSG_HOST = 'Host: %s'
MSG_USER = 'User: %s'
MSG_ITEM = '  [%d] | %s'
MSG_SUB_MENU = 'Go to %s'
MSG_QUIT = 'Quit'
MSG_RETURN = 'Return to %s'
MSG_INPUT_NUM = 'Press menu number (%d-%d): '
MSG_INPUT_ANY = 'Press any key to continue...'
MSG_CONFIRM_TITLE = 'Confirmation'
MSG_CONFIRM = '  Would execute: %s'
MSG_CONFIRM_QUESTION = 'Do you really want to execute? (y/n) [n]: '
MSG_RUN_TITLE = 'Executing: %s'
